<?php
abstract class AbstractGatewayForwarder {
    protected $name;
    public $app_base_url = 'https://wishlistproducts.com/';
    public function __construct() {
        $this->name = get_class($this);
    }
    /**
     * Renders the requested form
     * @param string $f the form
     */
    abstract function handle();
    /**
     * Creates a unique session based on settings passed
     * @param mixed $settings
     * @return string the identifier for the session
     */
    public function create_session($settings) {
        $auth = md5(serialize($settings) . microtime() );
        $fp = fopen("tmp/$auth", 'w');
        if(!$fp) {
            WLMGatewayDebug::write("Cannot initialize sessions, make sure tmp is writeable");
            throw new Exception("Could not create session");
        }
        fwrite($fp, serialize($settings));
        fclose($fp);
        return $auth;
    }
    /**
     * Clears the session
     * @param string $auth the session identifier
     */
    public function clear_session($auth = '') {
        if(file_exists("tmp/$auth")) {
            unlink("tmp/$auth");
        }
    }
    /**
     * Retrieves the settings for the particular session
     * @param string $auth the session identifier
     * @return array
     */
    public function get_session_settings($auth = '') {
        if(empty($auth)){
            return array();
        }
        if(!file_exists("tmp/$auth")) {
            return false;
        }
        $s = file_get_contents("tmp/$auth");
        return unserialize($s);
    }
    /**
     * Builds the auth URL
     * @param String $auth
     */
    public function get_auth_url($auth, $query_append = '') {
        return sprintf("%sindex.php?t=%s&auth=%s%s", $this->app_base_url, get_class($this),
                $auth, $query_append);
    }

    public function set_app_base_url($url) {
        $this->app_base_url = $url;
    }
}