<?php
class EwayRecurringGatewayProcessor extends EwayPaymentProcessor {
    public function __construct($auth, $settings = array(), $forwarder = null) {
        parent::__construct($auth, $settings, $forwarder);
    }
    public function process() {
        extract($this->settings);
        $post_url = $this->forwarder->get_auth_url($this->session_auth, '&eway_t='.$this->settings['eway_t'].'&m=send');
        $method = 'showform';
        $method = (isset($_GET['m']) && !empty($_GET['m']))? $_GET['m'] : $method;

        switch($method) {
            case 'showform':
                WLMGatewayDebug::write("Form displayed", $this->session_auth);
                $first_name     = "";
                $last_name      = "";
                $email          = "";
                $address        = "";
                $city           = "";
                $state          = "";
                $postal_code    = "";
                $country        = "";
                $cc_name        = "";
                $cc_number      = "";
                $cc_exp_month   = "";
                $cc_exp_year    = "";
                include 'forms/eway_rebill_form.php';
                break;
            case 'check':
                $svc = new Eway24WebserviceClient($this->eway_24_test_gateway,
                    $this->settings['customer_id'],
                    $this->settings['username'],
                    $this->settings['password'], $this->is_test);

                $tid = $this->settings['transaction_id'];
                list($tmp, $cust_id, $rebill_id, $ref_id) = explode("-", $tid);

                $params = array(
                    'ewayCustomerInvoiceRef' => $ref_id,
                );
                $res = $svc->call("Transaction24HourReportByInvoiceReference", $params);
                if(!$res) {
                    echo 'No Response from Gateway';
                    //do not do anything stupid
                    return;
                }
                $res = $res['Transaction24HourReportByInvoiceReferenceResult'];

                if(isset($res['ewayTrxnStatus'])
                        && strtolower($res['ewayTrxnStatus']) == 'true') {
                    //reactivate
                    $data['cmd']        = 'ACTIVATE';
                    $data['transaction_id'] = $tid;
                    $data['hash']       =   md5('ACTIVATE__' . $this->settings['generic_secret']
                            . '__' . strtoupper(implode('|', $data)));
                    $request = new Curl();
                    $response = $request->simple_post($this->settings['wlm_postback'], $data);
                } else {
                    //deactivate
                    $data['cmd']        = 'DEACTIVATE';
                    $data['transaction_id'] = $tid;
                    $data['hash']       =   md5('DEACTIVATE__' . $this->settings['generic_secret']
                            . '__' . strtoupper(implode('|', $data)));
                    $request = new Curl();
                    $response = $request->simple_post($this->settings['wlm_postback'], $data);
                }
                //cleanup
                $this->clear_session();
                break;
            case 'send':
                $err = "";
                $first_name     = trim($_POST['first_name']);
                $last_name      = trim($_POST['last_name']);
                $email          = trim($_POST['email']);
                $address        = trim($_POST['address']);
                $city           = trim($_POST['city']);
                $state          = trim($_POST['state']);
                $postal_code    = trim($_POST['postal_code']);
                $country        = trim($_POST['country']);
                $cc_name        = trim($_POST['cc_name']);
                $cc_number      = trim($_POST['cc_number']);
                $cc_exp_month   = trim($_POST['cc_exp_month']);
                $cc_exp_year    = trim($_POST['cc_exp_year']);
                $end_date       = date('d/m/Y', strtotime($this->settings['rebill_end_date']));
                $start_date     = date('d/m/Y');

                //server side validations
                if(empty($first_name)) {
                    $err .= "<li>First Name should not be empty</li>";
                }
                if(empty($last_name)) {
                    $err .= "<li>Last Name should not be empty</li>";
                }
                if(empty($email)) {
                    $err .= "<li>Email Address should not be empty</li>";
                } elseif(!preg_match("/^([a-z0-9\+_\-]+)(\.[a-z0-9\+_\-]+)*@([a-z0-9\-]+\.)+[a-z]{2,6}$/ix", $email)) {
                    $err .= "<li>Please enter a valid email dddress</li>";
                }
                if(empty($state)) {
                    $err .= "<li>State Should not be empty</li>";
                }
                if(empty($postal_code)) {
                    $err .= "<li>Postal Code should not be empty</li>";
                }
                if(empty($country)) {
                    $err .= "<li>Country should not be empty</li>";
                }
                if(empty($cc_name)) {
                    $err .= "<li>Name on Credit Card should not be empty</li>";
                }

                if(empty($cc_number)) {
                    $err .= "<li>Credit Card Number should not be empty</li>";
                }

                $ws = new EwayRecurWebserviceClient($this->settings['customer_id'],
                        $this->settings['username'], $this->settings['password'], $this->is_test);

                if(!empty($err)) {
                    //don't waste time with a request if we have errors in
                    include 'forms/eway_rebill_form.php';
                    return;
                }

                //create the cust
                $data['customerTitle']      = '';
                $data['customerFirstName']  = $first_name;
                $data['customerLastName']   = $last_name;
                $data['customerAddress']    = $address;
                $data['customerSuburb']     = '';
                $data['customerState']      = $state;
                $data['customerCompany']    = '';
                $data['customerPostCode']   = $postal_code;
                $data['customerCountry']    = $country;
                $data['customerEmail']      = $email;
                $data['customerFax']        = '';
                $data['customerPhone1']     = '';
                $data['customerPhone2']     = '';
                $data['customerRef']        = '';
                $data['customerJobDesc']    = '';
                $data['customerComments']   = '';
                $data['customerURL']        = '';
                $res = $ws->call('CreateRebillCustomer', $data);
                WLMGatewayDebug::write("resp: " . serialize($res), $this->session_auth);
                if(!strtolower($res['CreateRebillCustomerResult']['Result']) ===
                        'success') {
                    WLMGatewayDebug::write("Customer creation failed: ".  serialize($res), $this->session_auth);
                    throw new Exception("Could not create rebill customer");
                }
                WLMGatewayDebug::write("Customer Created", $this->session_auth);

                //now create the rebill
                //create an invoiceRef in order to track this txn
                //we need a txn id when querying the 24 hour txn api
                //note: eway gateway doesn't like non-numeric invoice ref
                $invoice_ref = microtime(true);
                $invoice_ref = preg_replace('/\D/','', $invoice_ref);
                $data = array();
                $data['RebillCustomerID']   = $res['CreateRebillCustomerResult']['RebillCustomerID'];
                $data['RebillInvRef']       = $invoice_ref;
                $data['RebillInvDes']       = '';
                $data['RebillCCName']       = $cc_name;
                $data['RebillCCNumber']     = $cc_number;
                $data['RebillCCExpMonth']   = $cc_exp_month;
                $data['RebillCCExpYear']    = $cc_exp_year;
                $data['RebillInitAmt']      = (int) $this->settings['rebill_init_amount'];
                $data['RebillInitDate']     = $start_date;
                $data['RebillRecurAmt']     = (int) $this->settings['rebill_recur_amount'];
                $data['RebillStartDate']    = $start_date;
                $data['RebillInterval']     = (int) $this->settings['rebill_interval'];
                $data['RebillIntervalType'] = (int) $this->settings['rebill_interval_type'];
                $data['RebillEndDate']      = $end_date;
                $res = $ws->call('CreateRebillEvent', $data);
                if(strtolower($res['CreateRebillEventResult']['Result'])
                        === 'success') {
                    WLMGatewayDebug::write("Rebill Event Created", $this->session_auth);
                    $data = array();
                    $rebill_cust_id         = $res['CreateRebillEventResult']['RebillCustomerID'];
                    $rebill_event_id        = $res['CreateRebillEventResult']['RebillID'];
                    $data['cmd']            = 'CREATE';
                    $data['firstname']      = $first_name;
                    $data['lastname']       = $last_name;
                    $data['username']       = $email;
                    $data['email']          = $email;
                    $data['level']          = $this->settings['wlm_sku'];
                    $data['transaction_id'] = sprintf('EWAYRB-%s-%s-%s', $rebill_cust_id, $rebill_event_id, $invoice_ref);
                    $data['hash']           =   md5('CREATE__' . $this->settings['generic_secret'] . '__' . strtoupper(implode('|', $data)));

                    $request = new Curl();
                    $response = $request->simple_post($this->settings['wlm_postback'], $data);
                    list($status, $url) = explode("\n", $response);
                    WLMGatewayDebug::write("Received status $status. Sent to $url for registration", $this->session_auth);
                    //cleanup
                    $this->clear_session();
                    header("Location: $url");
                } else {
                    WLMGatewayDebug::write("Rebill creation failed: ".  serialize($res), $this->session_auth);
                    $err = "<li>Invalid Credit Card Details</li>";
                    include 'forms/eway_rebill_form.php';
                }
                break;
            default:
                break;
        }

    }
}