<?php

if (!class_exists('WP_Http')) {
	include_once( ABSPATH . WPINC . '/class-http.php' );
}

//$__classname__ = 'WLM_INTEGRATION_EWAY';
//$__optionname__ = 'eway_regurl';
//$__methodname__ = 'delegate';
if (!class_exists('Abstract_Eway')) {

	abstract class AbstractEwayIntegrationProcessor {

		static $wlm_gateway_connector_url = 'https://wishlistproducts.loc/';
		public $wlm = null;
		public $reg_url = "";
		public $cancel_url = "";
		public $customer_id = "";
		public $password = "";
		public $username = "";
		public $currency = "AUD";
		public $page_title = "";
		public $page_description = "";
		public $page_footer = "";
		public $sku = "";
		public $amounts = array();
		public $company_name = "";
		public $company_logo = "";
		public $test_mode = "";
		public $page_banner = "";
		public $invoice_desc = "";

		function __construct($that) {
			$this->wlm = $that;
			$this->reg_url = $that->GetOption('eway_regurl');
			$this->cancel_url = $that->GetOption('eway_cancel_url');
			$this->customer_id = $that->GetOption('eway_customer_id');
			$this->username = $that->GetOption('eway_username');
			$this->password = $that->GetOption('eway_password');

			$currency = $that->GetOption('eway_currency');
			if (!empty($currency)) {
				$this->currency = $currency;
			}

			$this->amounts = $that->GetOption('eway_amount');
			$this->page_title = $that->GetOption('eway_page_title');
			$this->page_description = $that->GetOption('eway_page_description');
			$this->page_footer = $that->GetOption('eway_page_footer');
			$this->page_banner = $that->GetOption('eway_page_banner');
			$this->company_name = $that->GetOption('eway_company_name');
			$this->company_log = $that->GetOption('eway_company_logo');
			$this->invoice_desc = $that->GetOption('eway_invoice_desc');
			$this->test_mode = $that->GetOption('eway_test_mode');
			self::$wlm_gateway_connector_url = $that->GetOption('eway_connector');
			/*
			 * @TODO: make sure there's a trailing slash or this will fail
			 * miserably
			 */
			if (empty(self::$wlm_gateway_connector_url)) {
				die("Cannot Find the eway connector, Please install it first");
			}
		}

		function set_sku($sku) {
			$this->sku = $sku;
		}

		abstract function process();
	}

}

if (!class_exists('EwayRecurringIntegrationProcessor')) {

	class EwayRecurringIntegrationProcessor extends AbstractEwayIntegrationProcessor {

		public $eway_t = 'recurring';

		function __construct($that) {
			parent::__construct($that);
		}

		function get_payment_url() {
			$wpm_levels = $this->wlm->GetOption('wpm_levels');
			$level = $wpm_levels[$this->sku];
			$postback = sprintf("%s/index.php/register/%s", get_bloginfo('url'), $this->wlm->GetOption('genericthankyou'));
			$cancel_postback = sprintf("%s/index.php/register/%s", get_bloginfo('url'), $this->wlm->GetOption('eway_regurl'));
			//enable simple shortcodes
			$page_title = str_replace('[level]', $level['name'], $this->page_title);
			$page_description = str_replace('[level]', $level['name'], $this->page_description);
			$page_footer = str_replace('[level]', $level['name'], $this->page_footer);

			$rebill_opts = $this->wlm->GetOption('eway_rebill_opts');
			$rebill_opts = $rebill_opts[$this->sku];

			$ewayurl.="?customer_id=$this->customer_id"
					. "&username=$this->username"
					. "&currency=$this->currency"
					. "&page_title=$page_title"
					. "&page_description=$page_description"
					. "&page_footer=$page_footer"
					. "&language="
					. "&company_name=$this->company_name"
					. "&invoice_description="
					. "&company_logo="
					. "&page_banner=$this->page_banner"
					. "&merchant_reference="
					. "&merchant_invoice="
					. "&merchange_option1="
					. "&merchange_option1="
					. "&merchange_option1="
					. "&rebill_init_amount=" . number_format($rebill_opts['rebill_init_amount'], 2)
					. "&rebill_recur_amount=" . number_format($rebill_opts['rebill_recur_amount'], 2)
					. "&rebill_interval=" . $rebill_opts['rebill_interval']
					. "&rebill_interval_type=" . $rebill_opts['rebill_interval_type']
					. "&rebill_end_date=" . $rebill_opts['rebill_end_date']
					. "&modifiable_customer_details=true"
					. "&wlm_postback=$postback"
					. "&wlm_cancel_postback=$cancel_postback"
					. "&wlm_sku=$this->sku"
					. "&t=EwayGatewayForwarder"
					. "&eway_t=$this->eway_t"
					. "&sess_start=true"
					. "&generic_secret=" . $this->wlm->GetOption('genericsecret')
					. "&test_mode=" . $this->test_mode
					. "&password=" . $this->password;

			$ewayurl = str_replace(" ", "%20", $ewayurl);
			$req = self::$wlm_gateway_connector_url . $ewayurl;
			$request = new WP_Http;
			$result = $request->request($req, array('sslverify' => false));
			if (isset($result['body']) && !empty($result['body'])) {
				$response = $result['body'];
				return json_decode($response);
			}
			return false;
		}

		static function status_check($init = false) {
			if ($init) {
				if (!wp_next_scheduled('wishlistmember_eway_recur_statuscheck')) {
					wp_schedule_event(time(), 'daily', array('wishlistmember_eway_recur_statuscheck', 'status_check'));
				}
				return;
			}
			global $wpdb;
			global $WishListMemberInstance;

			$customer_id = $WishListMemberInstance->GetOption('eway_customer_id');
			$username = $WishListMemberInstance->GetOption('eway_username');
			$password = $WishListMemberInstance->GetOption('eway_password');
			$test_mode = $WishListMemberInstance->GetOption('eway_test_mode');
			$postback = sprintf("%s/index.php/register/%s", get_bloginfo('url'), $WishListMemberInstance->GetOption('genericthankyou'));

			$order_ids = (array) $wpdb->get_col($q = "SELECT DISTINCT `option_value` FROM `{$WishListMemberInstance->Tables->userlevel_options}` WHERE `option_name`='transaction_id' AND `option_value` REGEXP '^EWAYRB'");
			$request = new WP_Http;
			$opts = array('sslverify' => false, 'timeout' => 60);

			foreach ($order_ids as $item) {
				$req = '';
				$req.="?customer_id=$customer_id"
						. "&username=$username"
						. "&wlm_postback=$postback"
						. "&t=EwayGatewayForwarder"
						. "&eway_t=recurring"
						. "&sess_start=true"
						. "&generic_secret=" . $WishListMemberInstance->GetOption('genericsecret')
						. "&password=" . $password
						. "&test_mode=" . $test_mode
						. "&transaction_id=" . $item;

				$req = str_replace(" ", "%20", $req);
				$req = self::$wlm_gateway_connector_url . $req;

				$result = $request->request($req, $opts);
				if (isset($result['body']) && !empty($result['body'])) {
					$response = $result['body'];
					$url = json_decode($response);
					$url .= "&m=check";
					$result = $request->request($url, $opts);
					echo $result['body'];
				}
				break;
			}
		}

		function process() {
			$do_init = true;
			if (isset($_GET['force_status_check'])) {
				$do_init = false;
			}
			self::status_check($do_init);

			if (isset($_GET['fwd']) && !empty($_GET['fwd'])) {
				$url = $this->get_payment_url();
				wp_redirect($url);
				die();
			}
		}

	}

}
if (!class_exists('EwaySharedIntegrationProcessor')) {

	class EwaySharedIntegrationProcessor extends AbstractEwayIntegrationProcessor {

		public $eway_t = 'shared';

		function __construct($that) {
			parent::__construct($that);
		}

		function process() {
			if (isset($_GET['fwd']) && !empty($_GET['fwd'])) {
				$url = $this->get_payment_url();
				wp_redirect($url);
				die();
			}
		}

		function get_payment_url() {
			$wpm_levels = $this->wlm->GetOption('wpm_levels');
			$level = $wpm_levels[$this->sku];
			$postback = sprintf("%s/index.php/register/%s", get_bloginfo('url'), $this->wlm->GetOption('genericthankyou'));
			$cancel_postback = sprintf("%s/index.php/register/%s", get_bloginfo('url'), $this->wlm->GetOption('eway_regurl'));
			//enable simple shortcodes
			$page_title = str_replace('[level]', $level['name'], $this->page_title);
			$page_description = str_replace('[level]', $level['name'], $this->page_description);
			$page_footer = str_replace('[level]', $level['name'], $this->page_footer);
			$invoice_desc = str_replace('[level]', $level['name'], $this->invoice_desc);

			$ewayurl.="?customer_id=$this->customer_id"
					. "&username=$this->username"
					. "&amount=" . number_format($this->amounts[$this->sku], 2)
					. "&currency=$this->currency"
					. "&page_title=$page_title"
					. "&page_description=$page_description"
					. "&page_footer=$page_footer"
					. "&language="
					. "&company_name=$this->company_name"
					. "&invoice_description=$invoice_desc"
					. "&company_logo=$this->company_logo"
					. "&page_banner=" . $this->page_banner
					. "&merchant_reference="
					. "&merchant_invoice="
					. "&merchange_option1="
					. "&merchange_option1="
					. "&merchange_option1="
					. "&modifiable_customer_details=true"
					. "&wlm_postback=$postback"
					. "&wlm_cancel_postback=$cancel_postback"
					. "&wlm_sku=$this->sku"
					. "&t=EwayGatewayForwarder"
					. "&eway_t=$this->eway_t"
					. "&sess_start=true"
					. "&test_mode=" . $this->test_mode
					. "&generic_secret=" . $this->wlm->GetOption('genericsecret');
			$ewayurl = str_replace(" ", "%20", $ewayurl);
			$req = self::$wlm_gateway_connector_url . $ewayurl;
			$request = new WP_Http;
			$result = $request->request($req, array('sslverify' => false));
			if (isset($result['body']) && !empty($result['body'])) {
				$response = $result['body'];
				return json_decode($response);
			}
			return false;
		}

	}

}

if (!class_exists('WLM_INTEGRATION_EWAY')) {

	class WLM_INTEGRATION_EWAY {

		function delegate($that) {
			$type = $_GET['eway_t'];
			if (empty($type)) {
				return;
			}

			$o = null;
			if ($type == 'recurring') {
				$o = new EwayRecurringIntegrationProcessor($that);
			} else {
				$o = new EwaySharedIntegrationProcessor($that);
			}
			$o->set_sku(wlm_arrval($_GET,'sku'));
			$o->process();
		}

	}

}

